/*
 * $Header: /opt/cvs/mumbletree/main.c,v 1.4 1999/10/22 08:44:29 morton Exp $
 *
 * $Log: main.c,v $
 * Revision 1.4  1999/10/22 08:44:29  morton
 * Added spillage optimisation
 *
 * Revision 1.2  1999/10/21 16:33:05  morton
 * *** empty log message ***
 *
 * Revision 1.1.1.1  1999/10/21 14:42:03  morton
 * First cut
 *
 */

#include <stdio.h>
#include <assert.h>
#include <stdarg.h>
#include <stdlib.h>

#include "mumbletree.h"

void fail(const char *str, ...)
{
	va_list ap;

	va_start(ap, str);
	vfprintf(stderr, str, ap);
	exit(1);
}

/*
 * Check that the tree matches the one described by expr
 *
 * Format of expr:
 *
 * (low,high,payload,left,right)
 *
 * eg:
 *            10,20,ro
 *      1,9,rw       21,25,rw
 *
 * is
 *
 * (10,20,ro,(1,9,rw,0,0),(21,25,rw,0,0))
 *
 * Return 0 on success
 */

int check_tree(mt_node *_mt, char *_expr)
{
	int success = 0;
	int retval = 0;
	char *expr = _expr;

	mt_node *wanted = mt_string_to_tree(&expr, &success);
	if (success != 0)
	{
		char *p = _expr;;
		fprintf(stderr, "error parsing expression\n");
		fprintf(stderr, "%s\n", _expr);
		while (*p && p < expr)
		{
			fputc(' ', stderr);
			p++;
		}
		fprintf(stderr, "^\n");
		exit(1);
	}
	if (success != 0)
	{
		retval = 1;
	}
	else
	{
		int cmp = mt_compare_trees(_mt, wanted);
		if (cmp != 0)
		{
			fprintf(stderr, "trees differ\n");
			fprintf(stderr, "current tree:\n");
			mt_print(_mt, stderr, -1, 0);
			fprintf(stderr, "wanted tree:\n");
			mt_print(wanted, stderr, -1, 0);
			retval = 1;
		}
	}
	mt_destroy_tree(wanted);
	return retval;
}

void exercise(const char *filename, mt_node *_mt)
{
	char buf[10000];
	int line = 0;

	FILE *f = fopen(filename, "r");
	if (f == 0)
		fail("Cannot open `%s'\n", filename);

	while (fgets(buf, sizeof(buf), f))
	{
		char command[1000];
		char s_payload[1000];
		char s_low[1000];
		char s_high[1000];
		int low, high = -1;
		mt_payload payload;
		int rv;

		if (buf[0] == '#')
			continue;

		++line;
		s_low[0] = s_high[0] = 0;
		rv = sscanf(buf, "%s %s %s %s", command, s_low, s_high, s_payload);
		low = atoi(s_low);
		high = atoi(s_high);
	
		switch (rv)
		{
		case 0:
		case -1:
			break;
		case 1:
			if (strcmp(command, "print") == 0)
				mt_print(_mt, stdout, -1, 0);
			else
				fail("Unrecognised command `%s'\n", command);
			break;
		case 2:
			if (strcmp(command, "assert") == 0)
			{
				if (check_tree(_mt, s_low) != 0)
				{
					fail("failed assertion\n");
				}
			}
			else
			{
				fail("Unrecognised command `%s'\n", command);
			}
			break;
		case 4:
			if (strcmp(s_payload, "nu") == 0)
				payload = mt_e_nouse;
			else if (strcmp(s_payload, "ro") == 0)
				payload = mt_e_readonly;
			else if (strcmp(s_payload, "rw") == 0)
				payload = mt_e_readwrite;
			else
				fail("unrecognised payload: %s\n", s_payload);

			if (strcmp(command, "set_right") == 0)
			{
				printf("Adding [%d,%d]->%s\n", low, high, s_payload);
				mt_set_range(_mt, low, high, payload);
			}
			else if (strcmp(command, "check_right") == 0)
			{
				mt_good_range(_mt, low, high, payload);
			}
			else
			{
				fail("Unrecognised command `%s'\n", command);
			}
			break;
		default:
			fail("garbage at line %d\n", line);
		}

	}
}

void usage()
{
	fprintf(stderr, "Usage: mumbletree testfile ...\n");
	exit(1);
}

int
main(int argc, char *argv[])
{
	int i;

	if (argc < 2)
	{
		usage();
	}

	for (i = 1; i < argc; i++)
	{
		mt_node *root_mt = mt_new_node();

		root_mt->low = 1;
		root_mt->high = 100;
		root_mt->left = 0;
		root_mt->right = 0;
		root_mt->payload = mt_e_nouse;

		exercise(argv[i], root_mt);
		mt_destroy_tree(root_mt);
	}

	return 0;
}
