/*
 * $Header: /opt/cvs/mumbletree/mumbletree.h,v 1.2 1999/10/22 08:12:30 morton Exp $
 *
 * $Log: mumbletree.h,v $
 * Revision 1.2  1999/10/22 08:12:30  morton
 * Made test dir, rewrote mumbletree core
 *
 * Revision 1.1.1.1  1999/10/21 14:42:03  morton
 * First cut
 *
 */

#include <stdio.h>

#include "mumbletree-conf.h"

typedef struct mt_node {
	mt_value low;		/* Low memory address */
	mt_value high;		/* High memory address */
	struct mt_node *left;	/* Child pointers */
	struct mt_node *right;
	mt_payload payload;	/* Contents of this node */
} mt_node;

/* Constructors */
mt_node *mt_new_node();

/* Destructor */
void mt_destroy_node(mt_node *);
void mt_destroy_tree(mt_node *);

mt_node *mt_set_range(	mt_node *_root,
			const mt_value _low,
			const mt_value _high,
			const mt_payload _payload);

/**
 * Check that the range at [_low, high) is of value _payload
 *
 * Return zero if there's a problem.
 */

int mt_good_range(	mt_node *_root,
			const mt_value _low,
			const mt_value _high,
			const mt_payload _payload);

/**
 * Print a tree
 *
 * If retval is non-zero, it's the error return from fprintf()
 * _depth is the depth to which to recur (use -1 for infinity)
 */

int mt_print(	mt_node *_root,
		FILE *_file,
		int _depth,
		int _spaces);

/*
 * Convert the passed string into a mumbletree.
 *
 * Format of expr:
 *
 * (low,high,payload,left,right)
 *
 * eg:
 *            10,20,ro
 *      1,9,rw       21,25,rw
 *
 * is
 *
 * (10,20,2,(1,9,3,0,0),(21,25,2,0,0))
 *
 * Return 0 in *success if things are OK.
 * Relies on the enumerated values of enum mt_payload.  Sorry.
 */

mt_node *mt_string_to_tree(char **_expr, int *_success);

/*
 * Compare trees t1 amd t2.
 * Return < 0 if t1 < t2
 * Return 0 if t1 == t2
 * Return > 0 if t1 > t2
 */
int mt_compare_trees(mt_node *_t1, mt_node *_t2);

